from enum import Enum


class GenerationsStyle(Enum):
    """
    Enums for generation style_ids
    """

    ANIME = 21
    PORTRAIT = 26
    IMAGINE_V1 = 27
    IMAGINE_V3 = 28
    REALISTIC = 29
    IMAGINE_V4 = 30
    IMAGINE_V4_CREATIVE = 31
    IMAGINE_V4_1 = 32
    IMAGINE_V5 = 33
    DELIBERATE = 100
    MAGIC_MIX = 101
    DISNEY = 102
    ABYSS_ORANGE_MIX = 103
    LYRIEL = 104
    RPG_4 = 105
    REALISTIC_VISION = 110
    DREAM_SHAPER = 111
    REV_ANIMATED = 112
    TOON_YOU = 113
    UNREAL_ENGINE_5 = 114
    EPIC_REALISM = 115
    MEINA_MIX = 116
    XX_MIX_9_REALISTIC = 117
    THREE_D_RENDERING = 118
    ABSOLUTE_REALITY_V1_6 = 119
